Class: RuboCop::AST::EnsureNode

Inherits:
Node
  • Object
show all
Defined in:
lib/rubocop/ast/node/ensure_node.rb

Overview

A node extension for ensure nodes. This will be used in place of a plain node when the builder constructs the AST, making its methods available to all ensure nodes within RuboCop.

Constant Summary

Constants inherited from Node

Node::ASSIGNMENTS, Node::BASIC_CONDITIONALS, Node::BASIC_LITERALS, Node::COMPARISON_OPERATORS, Node::COMPOSITE_LITERALS, Node::CONDITIONALS, Node::EQUALS_ASSIGNMENTS, Node::FALSEY_LITERALS, Node::IMMUTABLE_LITERALS, Node::KEYWORDS, Node::LITERALS, Node::LOOP_TYPES, Node::MUTABLE_LITERALS, Node::OPERATOR_KEYWORDS, Node::POST_CONDITION_LOOP_TYPES, Node::REFERENCES, Node::SHORTHAND_ASSIGNMENTS, Node::SPECIAL_KEYWORDS, Node::TRUTHY_LITERALS, Node::VARIABLES

Instance Method Summary collapse

Methods inherited from Node

#ancestors, #any_block_type?, #any_def_type?, #any_match_pattern_type?, #any_str_type?, #any_sym_type?, #argument?, #argument_type?, #assignment?, #assignment_or_similar?, #basic_conditional?, #basic_literal?, #boolean_type?, #call_type?, #chained?, #class_constructor?, #class_definition?, #complete!, #complete?, #conditional?, #const_name, #defined_module, #defined_module_name, #each_ancestor, #empty_source?, #equals_asgn?, #falsey_literal?, #first_line, #global_const?, #guard_clause?, #immutable_literal?, #initialize, #keyword?, #lambda?, #lambda_or_proc?, #last_line, #left_sibling, #left_siblings, #line_count, #literal?, #loc?, #loc_is?, #loop_keyword?, #match_guard_clause?, #module_definition?, #multiline?, #mutable_literal?, #nonempty_line_count, #numeric_type?, #operator_keyword?, #parent, #parent?, #parent_module_name, #parenthesized_call?, #post_condition_loop?, #proc?, #pure?, #range_type?, #receiver, #recursive_basic_literal?, #recursive_literal?, #reference?, #right_sibling, #right_siblings, #root?, #send_type?, #shorthand_asgn?, #sibling_index, #single_line?, #source, #source_length, #source_range, #special_keyword?, #str_content, #struct_constructor?, #truthy_literal?, #type?, #updated, #value_used?, #variable?

Methods included from NodePattern::Macros

#def_node_matcher, #def_node_search

Methods included from Descendence

#child_nodes, #descendants, #each_child_node, #each_descendant, #each_node

Methods included from Sexp

#s

Constructor Details

This class inherits a constructor from RuboCop::AST::Node

Instance Method Details

#bodyNode?

Deprecated.

Use ‘EnsureNode#branch`

Returns the body of the ensure clause.

Returns:

  • (Node, nil)

    The body of the ensure.



16
17
18
19
20
21
22
23
24
25
26
27
28
# File 'lib/rubocop/ast/node/ensure_node.rb', line 16

def body
  first_caller = caller(1..1).first

  unless DEPRECATION_WARNING_LOCATION_CACHE.include?(first_caller)
    warn '`EnsureNode#body` is deprecated and will be changed in the next major version of ' \
         'rubocop-ast. Use `EnsureNode#branch` instead to get the body of the `ensure` branch.'
    warn "Called from:\n#{caller.join("\n")}\n\n"

    DEPRECATION_WARNING_LOCATION_CACHE << first_caller
  end

  branch
end

#branchNode?

Returns an the ensure branch in the exception handling statement.

Returns:

  • (Node, nil)

    the body of the ensure branch.



33
34
35
# File 'lib/rubocop/ast/node/ensure_node.rb', line 33

def branch
  node_parts[1]
end

#rescue_nodeNode?

Returns the rescue node of the ensure, if present.

Returns:

  • (Node, nil)

    The rescue node.



40
41
42
# File 'lib/rubocop/ast/node/ensure_node.rb', line 40

def rescue_node
  node_parts[0] if node_parts[0].rescue_type?
end

#void_context?true

Checks whether this node body is a void context. Always true for ensure.

Returns:

  • (true)

    whether the ensure node body is a void context



48
49
50
# File 'lib/rubocop/ast/node/ensure_node.rb', line 48

def void_context?
  true
end