Module: RuboCop::AST::Traversal::CallbackCompiler Private
- Included in:
- RuboCop::AST::Traversal
- Defined in:
- lib/rubocop/ast/traversal.rb
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
Constant Summary collapse
- SEND =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
'send(TYPE_TO_METHOD[child.type], child)'- TEMPLATE =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
How a particular child node should be visited. For example, if a child node can be nil it should be guarded behind a nil check. Or, if a child node is a literal (like a symbol) then the literal itself should not be visited.
{ skip: '', always: code, nil?: "#{code} if child" }.freeze